@@ -1,4 +1,4 @@
-# $Id: Expr.pm,v 1.11 2000/02/28 10:40:21 matt Exp $
+# $Id: Expr.pm,v 1.13 2000/05/08 13:08:01 matt Exp $
 
 package XML::XPath::Expr;
 use XML::XPath::Function;
@@ -111,11 +111,14 @@ sub op_eval {
 					return op_div($node, $self->{lhs}, $self->{rhs});
 				};
 		/^mod$/	&& do {
-					return op_($node, $self->{lhs}, $self->{rhs});
+					return op_mod($node, $self->{lhs}, $self->{rhs});
 				};
 		/^\*$/	&& do {
 					return op_mult($node, $self->{lhs}, $self->{rhs});
 				};
+		/^\|$/	&& do {
+					return op_union($node, $self->{lhs}, $self->{rhs});
+				};
 		
 		die "No such operator, or operator unimplemented in ", $self->as_string, "\n";
 	}
@@ -159,8 +162,7 @@ sub op_equals {
 		# on the string-values of the two nodes is true.
 		foreach my $lhnode ($lh_results->get_nodelist) {
 			foreach my $rhnode ($rh_results->get_nodelist) {
-				if (XML::XPath::XMLParser::string_value($lhnode) eq
-						XML::XPath::XMLParser::string_value($rhnode)) {
+				if ($lhnode->string_value eq $rhnode->string_value) {
 					return XML::XPath::Boolean->True;
 				}
 			}
@@ -265,10 +267,8 @@ sub op_ge {
 
 		foreach my $lhnode ($lh_results->get_nodelist) {
 			foreach my $rhnode ($rh_results->get_nodelist) {
-				my $lhNum = XML::XPath::Number->new(
-						XML::XPath::XMLParser::string_value($lhnode));
-				my $rhNum = XML::XPath::Number->new(
-						XML::XPath::XMLParser::string_value($rhnode));
+				my $lhNum = XML::XPath::Number->new($lhnode->string_value);
+				my $rhNum = XML::XPath::Number->new($rhnode->string_value);
 				if ($lhNum->value >= $rhNum->value) {
 					return XML::XPath::Boolean->True;
 				}
@@ -363,10 +363,8 @@ sub op_gt {
 
 		foreach my $lhnode ($lh_results->get_nodelist) {
 			foreach my $rhnode ($rh_results->get_nodelist) {
-				my $lhNum = XML::XPath::Number->new(
-						XML::XPath::XMLParser::string_value($lhnode));
-				my $rhNum = XML::XPath::Number->new(
-						XML::XPath::XMLParser::string_value($rhnode));
+				my $lhNum = XML::XPath::Number->new($lhnode->string_value);
+				my $rhNum = XML::XPath::Number->new($rhnode->string_value);
 				if ($lhNum->value > $rhNum->value) {
 					return XML::XPath::Boolean->True;
 				}
@@ -529,4 +527,26 @@ sub op_mult {
 	return XML::XPath::Number->new($result);
 }
 
+sub op_union {
+	my ($node, $lhs, $rhs) = @_;
+	my $lh_result = $lhs->evaluate($node);
+	my $rh_result = $rhs->evaluate($node);
+	
+	if ($lh_result->isa('XML::XPath::NodeSet') &&
+			$rh_result->isa('XML::XPath::NodeSet')) {
+		my %found;
+		my $results = XML::XPath::NodeSet->new;
+		foreach my $lhnode ($lh_result->get_nodelist) {
+			$found{"$lhnode"}++;
+			$results->push($lhnode);
+		}
+		foreach my $rhnode ($rh_result->get_nodelist) {
+			$results->push($rhnode)
+					unless exists $found{"$rhnode"};
+		}
+		return $results;
+	}
+	die "Both sides of a union must be Node Sets\n";
+}
+
 1;
